@@ -142,6 +142,16 @@ class Forum(models.Model):
 
 
 class Topic(models.Model):
+    POLL_TYPE_NONE = 0
+    POLL_TYPE_SINGLE = 1
+    POLL_TYPE_MULTIPLE = 2
+
+    POLL_TYPE_CHOICES = (
+        (POLL_TYPE_NONE, _('None')),
+        (POLL_TYPE_SINGLE, _('Single answer')),
+        (POLL_TYPE_MULTIPLE, _('Multiple answers')),
+    )
+
     forum = models.ForeignKey(Forum, related_name='topics', verbose_name=_('Forum'))
     name = models.CharField(_('Subject'), max_length=255)
     created = models.DateTimeField(_('Created'), null=True)
@@ -150,15 +160,13 @@ class Topic(models.Model):
     views = models.IntegerField(_('Views count'), blank=True, default=0)
     sticky = models.BooleanField(_('Sticky'), blank=True, default=False)
     closed = models.BooleanField(_('Closed'), blank=True, default=False)
-    subscribers = models.ManyToManyField(
-        User,
-        related_name='subscriptions',
-        verbose_name=_('Subscribers'),
-        blank=True
-    )
+    subscribers = models.ManyToManyField(User, related_name='subscriptions', verbose_name=_('Subscribers'),
+        blank=True)
     post_count = models.IntegerField(_('Post count'), blank=True, default=0)
     readed_by = models.ManyToManyField(User, through='TopicReadTracker', related_name='readed_topics')
     on_moderation = models.BooleanField(_('On moderation'), default=False)
+    poll_type = models.IntegerField(_('Poll type'), choices=POLL_TYPE_CHOICES, default=POLL_TYPE_NONE)
+    poll_question = models.TextField(_('Poll question'), blank=True, null=True)
 
     class Meta(object):
         ordering = ['-created']
@@ -205,6 +213,12 @@ class Topic(models.Model):
         """
         return self.forum.category, self.forum
 
+    def poll_votes(self):
+        if self.poll_type != self.POLL_TYPE_NONE:
+            return PollAnswerUser.objects.filter(poll_answer__topic=self).count()
+        else:
+            return None
+
 
 class RenderableItem(models.Model):
     """
@@ -225,6 +239,7 @@ class RenderableItem(models.Model):
         # Unescape entities which was generated with the markup processor
         self.body_text = unescape(text)
 
+
 class Post(RenderableItem):
     topic = models.ForeignKey(Topic, related_name='posts', verbose_name=_('Topic'))
     user = models.ForeignKey(User, related_name='posts', verbose_name=_('User'))
@@ -285,6 +300,7 @@ class Post(RenderableItem):
         """
         return self.topic.forum.category, self.topic.forum, self.topic,
 
+
 class PybbProfile(models.Model):
     """
     Abstract class for user profile, site profile should be inherted from this class
@@ -378,6 +394,7 @@ class TopicReadTracker(models.Model):
     topic = models.ForeignKey(Topic, blank=True, null=True)
     time_stamp = models.DateTimeField(auto_now=True)
 
+
 class ForumReadTracker(models.Model):
     """
     Save per user forum read tracking
@@ -390,6 +407,40 @@ class ForumReadTracker(models.Model):
     forum = models.ForeignKey(Forum, blank=True, null=True)
     time_stamp = models.DateTimeField(auto_now=True)
 
-from pybb import signals
 
+class PollAnswer(models.Model):
+    topic = models.ForeignKey(Topic, related_name='poll_answers', verbose_name=_('Topic'))
+    text = models.CharField(max_length=255, verbose_name=_('Text'))
+
+    class Meta:
+        verbose_name = _('Poll answer')
+        verbose_name_plural = _('Polls answers')
+
+    def __unicode__(self):
+        return self.text
+
+    def votes(self):
+        return self.users.count()
+
+    def votes_percent(self):
+        topic_votes = self.topic.poll_votes()
+        if topic_votes > 0:
+            return 1.0 * self.votes() / topic_votes * 100
+
+
+class PollAnswerUser(models.Model):
+    poll_answer = models.ForeignKey(PollAnswer, related_name='users', verbose_name=_('Poll answer'))
+    user = models.ForeignKey(User, related_name='poll_answers', verbose_name=_('User'))
+    timestamp = models.DateTimeField(auto_now_add=True)
+
+    class Meta:
+        verbose_name = _('Poll answer user')
+        verbose_name_plural = _('Polls answers users')
+        unique_together = (('poll_answer', 'user', ), )
+
+    def __unicode__(self):
+        return u'%s - %s' % (self.poll_answer.topic, self.user)
+
+
+from pybb import signals
 signals.setup_signals()
