@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 
 import time
+import os
 
-from django.contrib.auth.models import User
+from django.contrib.auth.models import User, Permission
 from django.core import mail
 from django.core.urlresolvers import reverse
-from django.test import TestCase
+from django.test import TransactionTestCase
 from django.test.client import Client
 
 try:
@@ -42,7 +43,7 @@ class SharedTestModule(object):
         return dict(html.fromstring(response.content).xpath('//form[@class="%s"]' % form)[0].form_values())
 
 
-class FeaturesTest(TestCase, SharedTestModule):
+class FeaturesTest(TransactionTestCase, SharedTestModule):
     def setUp(self):
         self.ORIG_PYBB_ENABLE_ANONYMOUS_POST = defaults.PYBB_ENABLE_ANONYMOUS_POST
         self.ORIG_PYBB_PREMODERATION = defaults.PYBB_PREMODERATION
@@ -119,9 +120,10 @@ class FeaturesTest(TestCase, SharedTestModule):
         values = self.get_form_values(response)
         values['body'] = 'new topic test'
         values['name'] = 'new topic name'
+        values['poll_type'] = 0
         response = self.client.post(add_topic_url, data=values, follow=True)
         self.assertEqual(response.status_code, 200)
-        self.assertContains(response, 'new topic test')
+        self.assertTrue(Topic.objects.filter(name='new topic name').exists())
 
     def test_post_deletion(self):
         post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
@@ -247,6 +249,7 @@ class FeaturesTest(TestCase, SharedTestModule):
         values = self.get_form_values(response)
         values['body'] = 'topic_3'
         values['name'] = 'topic_3'
+        values['poll_type'] = 0
         response = client_ann.post(add_topic_url, data=values, follow=True)
         self.assertEqual(TopicReadTracker.objects.all().count(), 2)
         self.assertEqual(TopicReadTracker.objects.filter(user=user_ann).count(), 2)
@@ -441,25 +444,28 @@ class FeaturesTest(TestCase, SharedTestModule):
 
     def test_edit_post(self):
         self.login_client()
-        response = self.client.get(reverse('pybb:edit_post', kwargs={'pk': self.post.id}))
+        edit_post_url = reverse('pybb:edit_post', kwargs={'pk': self.post.id})
+        response = self.client.get(edit_post_url)
         self.assertEqual(response.status_code, 200)
         tree = html.fromstring(response.content)
         values = dict(tree.xpath('//form[@method="post"]')[0].form_values())
         values['body'] = 'test edit'
-        response = self.client.post(reverse('pybb:edit_post', kwargs={'pk': self.post.id}), data=values, follow=True)
+        response = self.client.post(edit_post_url, data=values, follow=True)
         self.assertEqual(response.status_code, 200)
+        self.assertEqual(Post.objects.get(pk=self.post.id).body, 'test edit')
         response = self.client.get(self.post.get_absolute_url(), follow=True)
         self.assertContains(response, 'test edit')
+
         # Check admin form
         self.user.is_staff = True
         self.user.save()
-        response = self.client.get(reverse('pybb:edit_post', kwargs={'pk': self.post.id}))
+        response = self.client.get(edit_post_url)
         self.assertEqual(response.status_code, 200)
         tree = html.fromstring(response.content)
         values = dict(tree.xpath('//form[@method="post"]')[0].form_values())
         values['body'] = 'test edit'
         values['login'] = 'new_login'
-        response = self.client.post(reverse('pybb:edit_post', kwargs={'pk': self.post.id}), data=values, follow=True)
+        response = self.client.post(edit_post_url, data=values, follow=True)
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, 'test edit')
 
@@ -554,7 +560,7 @@ class FeaturesTest(TestCase, SharedTestModule):
 
 
 
-class AnonymousTest(TestCase, SharedTestModule):
+class AnonymousTest(TransactionTestCase, SharedTestModule):
 
     def setUp(self):
         self.ORIG_PYBB_ENABLE_ANONYMOUS_POST = defaults.PYBB_ENABLE_ANONYMOUS_POST
@@ -562,12 +568,11 @@ class AnonymousTest(TestCase, SharedTestModule):
         defaults.PYBB_ENABLE_ANONYMOUS_POST = True
         defaults.PYBB_ANONYMOUS_USERNAME = 'Anonymous'
         self.user = User.objects.create_user('Anonymous', 'Anonymous@localhost', 'Anonymous')
-        self.category = Category(name='foo')
-        self.category.save()
-        self.forum = Forum(name='xfoo', description='bar', category=self.category)
-        self.forum.save()
-        self.topic = Topic(name='etopic', forum=self.forum, user=self.user)
-        self.topic.save()
+        self.category = Category.objects.create(name='foo')
+        self.forum = Forum.objects.create(name='xfoo', description='bar', category=self.category)
+        self.topic = Topic.objects.create(name='etopic', forum=self.forum, user=self.user)
+        add_post_permission = Permission.objects.get_by_natural_key('add_post', 'pybb', 'post')
+        self.user.user_permissions.add(add_post_permission)
 
     def test_anonymous_posting(self):
         post_url = reverse('pybb:add_post', kwargs={'topic_id': self.topic.id})
@@ -593,7 +598,7 @@ def premoderate_test(user, post):
         return True
     return False
 
-class PreModerationTest(TestCase, SharedTestModule):
+class PreModerationTest(TransactionTestCase, SharedTestModule):
 
     def setUp(self):
         self.ORIG_PYBB_PREMODERATION = defaults.PYBB_PREMODERATION
@@ -677,6 +682,7 @@ class PreModerationTest(TestCase, SharedTestModule):
         values = self.get_form_values(response)
         values['body'] = 'new topic test'
         values['name'] = 'new topic name'
+        values['poll_type'] = 0
         response = self.client.post(add_topic_url, values, follow=True)
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, 'new topic test')
@@ -702,7 +708,7 @@ class PreModerationTest(TestCase, SharedTestModule):
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
 
         
-class AttachmentTest(TestCase, SharedTestModule):
+class AttachmentTest(TransactionTestCase, SharedTestModule):
     def setUp(self):
         self.PYBB_ATTACHMENT_ENABLE = defaults.PYBB_ATTACHMENT_ENABLE
         defaults.PYBB_ATTACHMENT_ENABLE = True
@@ -721,13 +727,141 @@ class AttachmentTest(TestCase, SharedTestModule):
         values['attachments-0-file'] = self.file
         response = self.client.post(add_post_url, values, follow=True)
         self.assertEqual(response.status_code, 200)
-        self.assertContains(response, 'test attachment')
+        self.assertTrue(Post.objects.filter(body='test attachment').exists())
 
     def tearDown(self):
         defaults.PYBB_ATTACHMENT_ENABLE = self.PYBB_ATTACHMENT_ENABLE
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
 
-class FiltersTest(TestCase, SharedTestModule):
+class PollTest(TransactionTestCase, SharedTestModule):
+    def setUp(self):
+        self.create_user()
+        self.create_initial()
+        self.PYBB_POLL_MAX_ANSWERS = defaults.PYBB_POLL_MAX_ANSWERS
+        defaults.PYBB_POLL_MAX_ANSWERS = 2
+
+    def test_poll_add(self):
+        add_topic_url = reverse('pybb:add_topic', kwargs={'forum_id': self.forum.id})
+        self.login_client()
+        response = self.client.get(add_topic_url)
+        values = self.get_form_values(response)
+        values['body'] = 'test poll body'
+        values['name'] = 'test poll name'
+        values['poll_type'] = 0 # poll_type = None, create topic without poll answers
+        values['poll_question'] = 'q1'
+        values['poll_answers-0-text'] = 'answer1'
+        values['poll_answers-1-text'] = 'answer2'
+        values['poll_answers-TOTAL_FORMS'] = 2
+        response = self.client.post(add_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        new_topic = Topic.objects.get(name='test poll name')
+        self.assertIsNone(new_topic.poll_question)
+        self.assertFalse(PollAnswer.objects.filter(topic=new_topic).exists()) # no answers here
+
+        values['name'] = 'test poll name 1'
+        values['poll_type'] = 1
+        values['poll_answers-0-text'] = 'answer1' # not enough answers
+        values['poll_answers-TOTAL_FORMS'] = 1
+        response = self.client.post(add_topic_url, values, follow=True)
+        self.assertFalse(Topic.objects.filter(name='test poll name 1').exists())
+
+        values['name'] = 'test poll name 1'
+        values['poll_type'] = 1
+        values['poll_answers-0-text'] = 'answer1' # too many answers
+        values['poll_answers-1-text'] = 'answer2'
+        values['poll_answers-2-text'] = 'answer3'
+        values['poll_answers-TOTAL_FORMS'] = 3
+        response = self.client.post(add_topic_url, values, follow=True)
+        self.assertFalse(Topic.objects.filter(name='test poll name 1').exists())
+
+        values['name'] = 'test poll name 1'
+        values['poll_type'] = 1 # poll type = single choice, create answers
+        values['poll_question'] = 'q1'
+        values['poll_answers-0-text'] = 'answer1' # two answers - what do we need to create poll
+        values['poll_answers-1-text'] = 'answer2'
+        values['poll_answers-TOTAL_FORMS'] = 2
+        response = self.client.post(add_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        new_topic = Topic.objects.get(name='test poll name 1')
+        self.assertEqual(new_topic.poll_question, 'q1')
+        self.assertEqual(PollAnswer.objects.filter(topic=new_topic).count(), 2)
+
+    def test_poll_edit(self):
+        edit_topic_url = reverse('pybb:edit_post', kwargs={'pk': self.post.id})
+        self.login_client()
+        response = self.client.get(edit_topic_url)
+        values = self.get_form_values(response)
+        values['poll_type'] = 1 # add_poll
+        values['poll_question'] = 'q1'
+        values['poll_answers-0-text'] = 'answer1'
+        values['poll_answers-1-text'] = 'answer2'
+        values['poll_answers-TOTAL_FORMS'] = 2
+        response = self.client.post(edit_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_type, 1)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_question, 'q1')
+        self.assertEqual(PollAnswer.objects.filter(topic=self.topic).count(), 2)
+
+        values = self.get_form_values(self.client.get(edit_topic_url))
+        values['poll_type'] = 2 # change_poll type
+        values['poll_question'] = 'q100' # change poll question
+        values['poll_answers-0-text'] = 'answer100' # change poll answers
+        values['poll_answers-1-text'] = 'answer200'
+        values['poll_answers-TOTAL_FORMS'] = 2
+        response = self.client.post(edit_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_type, 2)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_question, 'q100')
+        self.assertEqual(PollAnswer.objects.filter(topic=self.topic).count(), 2)
+        self.assertTrue(PollAnswer.objects.filter(text='answer100').exists())
+        self.assertTrue(PollAnswer.objects.filter(text='answer200').exists())
+        self.assertFalse(PollAnswer.objects.filter(text='answer1').exists())
+        self.assertFalse(PollAnswer.objects.filter(text='answer2').exists())
+
+        values['poll_type'] = 0 # remove poll
+        values['poll_answers-0-text'] = 'answer100' # no matter how many answers we provide
+        values['poll_answers-TOTAL_FORMS'] = 1
+        response = self.client.post(edit_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_type, 0)
+        self.assertIsNone(Topic.objects.get(id=self.topic.id).poll_question)
+        self.assertEqual(PollAnswer.objects.filter(topic=self.topic).count(), 0)
+
+    def test_poll_voting(self):
+        def recreate_poll(poll_type):
+            self.topic.poll_type = poll_type
+            self.topic.save()
+            PollAnswer.objects.filter(topic=self.topic).delete()
+            PollAnswer.objects.create(topic=self.topic, text='answer1')
+            PollAnswer.objects.create(topic=self.topic, text='answer2')
+
+        self.login_client()
+        recreate_poll(poll_type=Topic.POLL_TYPE_SINGLE)
+        vote_url = reverse('pybb:topic_poll_vote', kwargs={'pk': self.topic.id})
+        my_answer = PollAnswer.objects.all()[0]
+        values = {'answers': my_answer.id}
+        response = self.client.post(vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Topic.objects.get(id=self.topic.id).poll_votes(), 1)
+        self.assertEqual(PollAnswer.objects.get(id=my_answer.id).votes(), 1)
+        self.assertEqual(PollAnswer.objects.get(id=my_answer.id).votes_percent(), 100.0)
+
+        # already voted
+        response = self.client.post(vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 400) # bad request status
+
+        recreate_poll(poll_type=Topic.POLL_TYPE_MULTIPLE)
+        values = {'answers': [a.id for a in PollAnswer.objects.all()]}
+        response = self.client.post(vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertListEqual([a.votes() for a in PollAnswer.objects.all()], [1, 1, ])
+        self.assertListEqual([a.votes_percent() for a in PollAnswer.objects.all()], [50.0, 50.0, ])
+
+    def tearDown(self):
+        defaults.PYBB_POLL_MAX_ANSWERS = self.PYBB_POLL_MAX_ANSWERS
+
+
+class FiltersTest(TransactionTestCase, SharedTestModule):
     def setUp(self):
         self.create_user()
         self.create_initial(post=False)
